; $Id: atm_gui.pro,v 7.1 2020/12/13 marcel.carbillet $
;+
; NAME:
;    atm_gui
;
; PURPOSE:
;    atm_gui generates the Graphical User Interface (GUI) FOR
;    setting the parameters of the ATMosphere (ATM) module.
;    A parameter file called atm_nnnnn.sav is created, where nnnnn
;    is the number n_module associated to the module instance.
;    The file is stored in the project directory proj_name located
;    in the working directory.
;    (see atm.pro's header --or file caos_help.html-- FOR details
;    about the module itself).
;
; CATEGORY:
;    module's Graphical User Interface routine
;
; CALLING SEQUENCE:
;    error = atm_gui(n_module, proj_name)
; 
; INPUTS:
;    n_module:  integer scalar. number associated to the intance
;               of the ATM module. n_module > 0.
;    proj_name: string. name of the current project.
;
; OUTPUTS:
;    error: long scalar, error code (see !caos_error var in caos_init.pro).
;
; COMMON BLOCKS:
;    none.
;
; CALLED NON-IDL FUNCTIONS:
;   sha_test
;   sha_test_gui
;   evol_time
;   trans_tab_gui 
;
; ROUTINE MODIFICATION HISTORY:
;    program written: october 1998,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it].
;    modIFications  : november 1998,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -atmosphere evolution time calculus modIFied
;                     in order to take into account the CASE where
;                     the layers' winds are zeros.
;                   : february 1999,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -a few modIFications in order to fit with version 1.0
;                     new features.
;                   : october 1999,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -the Fried parameter r0, the maximum field-of-view, the
;                     evolution time of turbulence tau0, as well as the r0
;                     sampling, are now given FOR all the pre-defined bands
;                     (Johnson and special Na) central wavelengths, with
;                     dedicated sub-GUIs.
;                   : november-april 2000,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -enhanced and adapted to version 2.0 (CAOS).
;                   : december 2000,
;                     Bruno Femenia (OAA) [bfemenia@arcetri.astro.it]:
;                    -parameter "zern_rad_degree" added.
;                   : june 2001,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -avoided the use of the astrolib routine TAG_EXIST
;                     (and subsEQuent compatibility with older versions of
;                      the module).
;                   : january/february 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -adapted to version 4.0 of the whole Software System CAOS
;                     (variable "pack_name" added, and variable "mod_type"
;                     changed into "mod_name").
;                    -(atm_info()).help stuff added (instead of !caos_env.help).
;                    -warning on r0 sampling eliminated (was almost always useless
;                     and confusing).
;                   : december 2004,
;                     Marcel Carbillet (LUAN) [marcel.carbillet@unice.fr]:
;                    -no more crash provoked when controlling the Soft.Pack.
;                     version FOR existing parameter files - just a warning.
;                   : february 2007,
;                     Marcel Carbillet (LUAN) [marcel.carbillet@unice.fr]:
;                    -control of the the Soft.Pack. debugged (the warning was
;                     here *always* (partly) PRINTed )...
;                   : april 2014,
;                     Marcel Carbillet (Lagrange) [marcel.carbillet@unice.fr]:
;                    -evolution time accessible even in the "statistical averaging" CASE.
;                   : october 2014,
;                     Marcel Carbillet (Fizeau) [marcel.carbillet@unice.fr]:
;                    -no more limit FOR the number of turbulent layers (but the one given
;                     by the default parameter file to be set up in atm_gen_default.pro).
;                   : march 2016,
;                     Ulysse Perruchon-Monge & Adama Sy (Dépt. Physique UNS),
;                     Marcel Carbillet (Lagrange) [marcel.carbillet@unice.fr]:
;                    -turbulence can now be switched off (by using par.turnatmos).
;                   : april 2016,
;                     Marcel Carbillet (Lagrange) [marcel.carbillet@unice.fr]:
;                    -adapted to Soft. Pack. CAOS 7.0.
;                   : may 2016,
;                     Marcel Carbillet (Lagrange) [marcel.carbillet@unice.fr],
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it]:
;                    -New way to call CAOS_HELP (by using the "online_help" 
;                     routine, indepENDent from the operating system used).
;                   : december 2020,
;                     Marcel Carbillet (Lagrange) [marcel.carbillet@unice.fr]:
;                    -GUI rearranged in order to fit smaller screens (TABs and other details).
;                    -smoother number-of-layers control.
;
;-
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; status setting procedure ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
PRO atm_gui_set, state

WIDGET_CONTROL, /HOURGLASS

CASE state.par.cal OF
   0: BEGIN
      WIDGET_CONTROL, state.id.evol_t,  /SENSITIVE
      t0=evol_time(state.par.r0, state.par.wind, state.par.weight)
      WIDGET_CONTROL, state.id.evol_t, SET_VALUE=t0
      state.par.method = 0
      WIDGET_CONTROL, state.id.method, SET_VALUE=state.par.method
      WIDGET_CONTROL, state.id.zern_rad_degree, SENSITIVE=0
   END
   1: BEGIN
      WIDGET_CONTROL, state.id.evol_t,  SENSITIVE=0
      WIDGET_CONTROL, state.id.evol_t,  SET_VALUE=0.
      WIDGET_CONTROL, state.id.t0_tab,  SENSITIVE=0
   END
ENDCASE

IF (state.par.model EQ 0) THEN WIDGET_CONTROL, state.id.L0, SENSITIVE=0 $
   ELSE WIDGET_CONTROL, state.id.L0, SENSITIVE=1

IF (state.par.n_layers gt state.par.max_layers) THEN BEGIN
   dummy = DIALOG_MESSAGE('Number of turbulent layers MUST BE <= '+strtrim(state.par.max_layers,2) $
                         +" (but one can upgrade it in atm_gen_default and restart with a new "    $
                         +"default parameters' file)", DIALOG_PARENT=atm_gui, TITLE='ATM error', /ERROR)
   state.par.n_layers = state.par.max_layers
   WIDGET_CONTROL, state.id.n_lay, SET_VALUE=state.par.n_layers
ENDIF

np = state.par.n_layers & atm_val = fltarr(4, np)
atm_val[0,*] = state.par.alt   [0:np-1]
atm_val[1,*] = state.par.weight[0:np-1]
atm_val[2,*] = state.par.wind  [0:np-1]
atm_val[3,*] = state.par.dir   [0:np-1]*!RADEG
WIDGET_CONTROL, state.id.atm, /DELETE_ROWS
WIDGET_CONTROL, state.id.atm, INSERT_ROWS=state.par.n_layers
WIDGET_CONTROL, state.id.atm, ROW_LABELS ="layer #"+strtrim(indgen(state.par.n_layers),2)
IF (state.par.cal EQ 1) THEN BEGIN
   atm_val[2, *] = 0 & atm_val[3, *] = 0
   WIDGET_CONTROL, state.id.atm, COLUMN_LABELS=["h [m]","Cn2 ratio"," "," "]
ENDIF ELSE WIDGET_CONTROL, state.id.atm, COLUMN_LABELS=["h [m]","Cn2 ratio","v [m/s]","dir [deg]"]

WIDGET_CONTROL, state.id.atm, SET_VALUE=atm_val

IF (state.par.length ne 0) THEN BEGIN
   WIDGET_CONTROL, state.id.sam_r0, SET_VALUE=state.par.r0*state.par.dim/state.par.length
   WIDGET_CONTROL, state.id.fov, SET_VALUE=5E-7*state.par.dim/state.par.length*!RADEG*3600.
ENDIF

IF (state.par.lps EQ 1) THEN BEGIN

   WIDGET_CONTROL, state.id.psg,     /SENSITIVE
   WIDGET_CONTROL, state.id.read_ps, SENSITIVE=0
   WIDGET_CONTROL, state.id.psg_L0,  SENSITIVE=0

   WIDGET_CONTROL, state.id.width,   SET_VALUE=state.par.length

ENDIF ELSE BEGIN

   WIDGET_CONTROL, state.id.psg,     SENSITIVE=0
   WIDGET_CONTROL, state.id.read_ps, /SENSITIVE
   WIDGET_CONTROL, state.id.psg_L0,  /SENSITIVE

   error = psg_openr_cube(unit, state.par.psg_add, header, /GET_LUN)
   FREE_LUN, unit

   IF (state.par.dim ne 0) THEN                               $
      WIDGET_CONTROL, state.id.psg_L0,    SET_VALUE=header.L0 $
                                             * state.par.length/header.dim_x

   IF (state.par.length ne 0) THEN BEGIN
      WIDGET_CONTROL, state.id.sam_r0,    SET_VALUE=state.par.r0 $
                                             * header.dim_x/state.par.length
      WIDGET_CONTROL, state.id.fov,       SET_VALUE=5E-7*header.dim_x $
                                                   /state.par.length*!RADEG*3600
   ENDIF

   WIDGET_CONTROL, state.id.psg_dimx,     SET_VALUE=header.dim_x
   WIDGET_CONTROL, state.id.psg_dimy,     SET_VALUE=header.dim_y
   WIDGET_CONTROL, state.id.psg_nscreens, SET_VALUE=header.n_screens
   WIDGET_CONTROL, state.id.psg_sha,      SET_VALUE=header.sha
   WIDGET_CONTROL, state.id.psg_seed1,    SET_VALUE=header.seed1
   WIDGET_CONTROL, state.id.psg_seed2,    SET_VALUE=header.seed2

   IF (header.method EQ 0) THEN dummy = "FFT (+ subharmonics)" $
   ELSE dummy = "Zernike polynomials"
   WIDGET_CONTROL, state.id.psg_meth,  SET_VALUE=dummy
   IF (header.model EQ 0) THEN dummy = "Kolmogorov" ELSE dummy = "von Karman"
   WIDGET_CONTROL, state.id.psg_model, SET_VALUE=dummy
   IF (header.DOuble EQ 1B) THEN dummy = "double" ELSE dummy = "floating"
   WIDGET_CONTROL, state.id.psg_prec,  SET_VALUE=dummy

   IF (header.method EQ 1) THEN BEGIN
      WIDGET_CONTROL, state.id.psg_sha,   SENSITIVE=0
      WIDGET_CONTROL, state.id.psg_seed2, SENSITIVE=0
   ENDIF ELSE BEGIN
      WIDGET_CONTROL, state.id.psg_sha,  /SENSITIVE
      IF (header.sha ne 0) THEN WIDGET_CONTROL, state.id.psg_seed2, /SENSITIVE $
      ELSE WIDGET_CONTROL, state.id.psg_seed2, SENSITIVE=0
   ENDELSE

   IF (header.model EQ 0) THEN WIDGET_CONTROL, state.id.psg_L0, SENSITIVE=0

   WIDGET_CONTROL, state.id.width, SET_VALUE=header.dim_y*state.par.length/header.dim_x

ENDELSE

CASE state.par.turnatmos OF
   0B: BEGIN
      state.par.n_layers = 1
      WIDGET_CONTROL, state.id.n_lay, SET_VALUE=state.par.n_layers
      state.par.sha = 0
      WIDGET_CONTROL, state.id.sha,   SET_VALUE=state.par.sha
      state.par.lps = 1
      WIDGET_CONTROL, state.id.lps,   SET_VALUE=state.par.lps
      
      WIDGET_CONTROL, state.id.cal,             SENSITIVE=0
      WIDGET_CONTROL, state.id.evol_t,          SENSITIVE=0
      WIDGET_CONTROL, state.id.t0_tab,          SENSITIVE=0
      WIDGET_CONTROL, state.id.n_lay,           SENSITIVE=0
      WIDGET_CONTROL, state.id.atm,             SENSITIVE=0
      WIDGET_CONTROL, state.id.r0,              SENSITIVE=0
      WIDGET_CONTROL, state.id.r0_tab,          SENSITIVE=0
      WIDGET_CONTROL, state.id.lps,             SENSITIVE=0
      WIDGET_CONTROL, state.id.sam_r0,          SENSITIVE=0
      WIDGET_CONTROL, state.id.sam_r0_tab,      SENSITIVE=0
      WIDGET_CONTROL, state.id.read_ps,         SENSITIVE=0
      WIDGET_CONTROL, state.id.method,          SENSITIVE=0
      WIDGET_CONTROL, state.id.model,           SENSITIVE=0
      WIDGET_CONTROL, state.id.seed1,           SENSITIVE=0
      WIDGET_CONTROL, state.id.seed2,           SENSITIVE=0
      WIDGET_CONTROL, state.id.sha_test,        SENSITIVE=0
      WIDGET_CONTROL, state.id.title_sha,       SENSITIVE=0
      WIDGET_CONTROL, state.id.rec_sha,         SENSITIVE=0
      WIDGET_CONTROL, state.id.sha,             SENSITIVE=0
      WIDGET_CONTROL, state.id.L0,              SENSITIVE=0
      WIDGET_CONTROL, state.id.zern_rad_degree, SENSITIVE=0       
   END
   1B: BEGIN
      WIDGET_CONTROL, state.id.cal,            /SENSITIVE
      WIDGET_CONTROL, state.id.evol_t,         /SENSITIVE
      WIDGET_CONTROL, state.id.t0_tab,         /SENSITIVE
      WIDGET_CONTROL, state.id.n_lay,          /SENSITIVE
      WIDGET_CONTROL, state.id.atm,            /SENSITIVE
      WIDGET_CONTROL, state.id.r0,             /SENSITIVE
      WIDGET_CONTROL, state.id.r0_tab,         /SENSITIVE
      WIDGET_CONTROL, state.id.lps,            /SENSITIVE
      WIDGET_CONTROL, state.id.sam_r0,         /SENSITIVE
      WIDGET_CONTROL, state.id.sam_r0_tab,     /SENSITIVE
      WIDGET_CONTROL, state.id.read_ps,        /SENSITIVE
      WIDGET_CONTROL, state.id.method,         /SENSITIVE
      WIDGET_CONTROL, state.id.model,          /SENSITIVE
      WIDGET_CONTROL, state.id.seed1,          /SENSITIVE
      WIDGET_CONTROL, state.id.seed2,          /SENSITIVE
      WIDGET_CONTROL, state.id.sha_test,       /SENSITIVE
      WIDGET_CONTROL, state.id.title_sha,      /SENSITIVE
      WIDGET_CONTROL, state.id.rec_sha,        /SENSITIVE
      WIDGET_CONTROL, state.id.sha,            /SENSITIVE
      WIDGET_CONTROL, state.id.L0,             /SENSITIVE
      WIDGET_CONTROL, state.id.zern_rad_degree,/SENSITIVE
   END
ENDCASE

IF (state.par.method EQ 1) THEN BEGIN
   state.par.model = 0
   WIDGET_CONTROL, state.id.model,           SET_VALUE=state.par.model
   WIDGET_CONTROL, state.id.sha,             SENSITIVE=0
   WIDGET_CONTROL, state.id.rec_sha,         SENSITIVE=0
   WIDGET_CONTROL, state.id.title_sha,       SENSITIVE=0
   WIDGET_CONTROL, state.id.sha_test,        SENSITIVE=0
   WIDGET_CONTROL, state.id.seed2,           SENSITIVE=0
   WIDGET_CONTROL, state.id.zern_rad_degree,/SENSITIVE
   WIDGET_CONTROL, state.id.zern_rad_degree, SET_VALUE=state.par.zern_rad_degree
ENDIF ELSE BEGIN
   WIDGET_CONTROL, state.id.sha,            /SENSITIVE
   WIDGET_CONTROL, state.id.rec_sha,        /SENSITIVE
   WIDGET_CONTROL, state.id.title_sha,      /SENSITIVE
   WIDGET_CONTROL, state.id.sha_test,       /SENSITIVE
   WIDGET_CONTROL, state.id.seed2,          /SENSITIVE
   WIDGET_CONTROL, state.id.zern_rad_degree, SENSITIVE=0
   IF (state.par.model EQ 1 AND state.par.length NE 0 AND state.par.L0 NE 0) $
   THEN BEGIN
      dummy = ceil(alog(state.par.L0/state.par.length/sqrt(.99^(-6/5.)-1))/alog(3))
      IF (dummy lt 0) THEN dummy=0
      WIDGET_CONTROL, state.id.rec_sha, SET_VALUE=dummy
   ENDIF ELSE WIDGET_CONTROL, state.id.rec_sha, SET_VALUE=9
ENDELSE

END

;;;;;;;;;;;;;;;;;;;
; GUI events loop ;
;;;;;;;;;;;;;;;;;;;
;
PRO atm_gui_event, event

COMMON error_block, error

; read the GUI state structure
WIDGET_CONTROL, event.top, GET_UVALUE=state

; handle a kill rEQuest (considered as a cancel event).
IF tag_names(event, /STRUCTURE_NAME) EQ 'WIDGET_KILL_REQUEST' THEN BEGIN
   error = !caos_error.cancel
   WIDGET_CONTROL, event.top, /DESTROY
ENDIF

; handle all the other events.
WIDGET_CONTROL, event.id, GET_UVALUE=uvalue
CASE uvalue OF

   'turnatmos': BEGIN
      state.par.turnatmos = event.value
      atm_gui_set, state
      WIDGET_CONTROL, event.top, SET_UVALUE=state
   END

   'menu_cal': BEGIN
      state.par.cal = event.value
      atm_gui_set, state
      WIDGET_CONTROL, event.top, SET_UVALUE=state
   END

   'delta_t': BEGIN
      state.par.delta_t = event.value
      atm_gui_set, state
      WIDGET_CONTROL, event.top, SET_UVALUE=state
   END

   't0_tab': BEGIN
      t0 = evol_time(state.par.r0, state.par.wind, state.par.weight)
      title = "ATM evolution time of turbulence GUI"
      sub_title = "tau0 [s]"
      dummy = trans_tab_gui(t0, 5E-7, 6/5., title, sub_title, GROUP_LEADER=event.top)
   END

   'r0' : BEGIN
      state.par.r0 = event.value
      atm_gui_set, state
      WIDGET_CONTROL, event.top, SET_UVALUE=state
   END

   'r0_tab': BEGIN
      title = "ATM Fried parameter GUI"
      sub_title = "r0 [m]"
      dummy = trans_tab_gui(state.par.r0, 5E-7, 6/5., title, sub_title, GROUP_LEADER=event.top)
   END

   'L0' : BEGIN
      state.par.L0 = event.value
      atm_gui_set, state
      WIDGET_CONTROL, event.top, SET_UVALUE=state
   END

   'n_lay': BEGIN
      state.par.n_layers = event.value
      atm_gui_set, state
      WIDGET_CONTROL, event.top, SET_UVALUE=state
   END

   'table_atm': IF (event.type EQ 0) THEN BEGIN
      WIDGET_CONTROL, event.id,  GET_VALUE =dummy
      np = state.par.n_layers
      state.par.alt    = 0 & state.par.alt   [0:np-1] = dummy[0,*]
      state.par.weight = 0 & state.par.weight[0:np-1] = dummy[1,*]
      state.par.wind   = 0 & state.par.wind  [0:np-1] = dummy[2,*]
      state.par.dir    = 0 & state.par.dir   [0:np-1] = dummy[3,*]/!RADEG
      atm_gui_set, state
      WIDGET_CONTROL, event.top,  SET_UVALUE=state
   ENDIF

   'sam_r0_tab': BEGIN
      title = "ATM Fried parameter sampling GUI"
      sub_title = "pixels/r0"
      sam_r0 = state.par.r0 * state.par.dim/state.par.length
      dummy = trans_tab_gui(sam_r0, 5E-7, 1., title, sub_title, GROUP_LEADER=event.top)
   END

   'fov_tab': BEGIN
      title = "ATM  maximum field-of-view GUI"
      sub_title = "FoV ['']"
      fov = 5E-7 * state.par.dim/state.par.length * 3600. * !RADEG
      dummy = trans_tab_gui(fov, 5E-7, 1., title, sub_title, GROUP_LEADER=event.top)
   END

   'menu_lps': BEGIN
      state.par.lps = event.value
      atm_gui_set, state
      WIDGET_CONTROL, event.top, SET_UVALUE=state
   END

   'psg_add': BEGIN
      state.par.psg_add = event.value
      atm_gui_set, state
      WIDGET_CONTROL, event.top, SET_UVALUE=state
   END

   'menu_meth': BEGIN
      state.par.method = event.value
      atm_gui_set, state
      WIDGET_CONTROL, event.top, SET_UVALUE=state
   END

   'dim': BEGIN
      state.par.dim = event.value
      atm_gui_set, state
      WIDGET_CONTROL, event.top, SET_UVALUE=state
   END

   'length': BEGIN
      state.par.length = event.value
      atm_gui_set, state
      WIDGET_CONTROL, event.top, SET_UVALUE=state
   END

   'menu_mod': BEGIN
      state.par.model = event.value
      atm_gui_set, state
      WIDGET_CONTROL, event.top, SET_UVALUE=state
   END

   'sha': BEGIN
      state.par.sha = event.value
      WIDGET_CONTROL, event.top, SET_UVALUE=state
   END

   'zern_rad_degree': BEGIN 
      state.par.zern_rad_degree = event.value
      WIDGET_CONTROL, event.top, SET_UVALUE=state
   END 

   'sha_test': BEGIN
      dummy = sha_test_gui(state.par.sha, state.par.model, state.par.length, state.par.L0, $
                           GROUP_LEADER=event.top)
      state.par.sha = dummy
      WIDGET_CONTROL, state.id.sha, SET_VALUE =state.par.sha
      WIDGET_CONTROL, event.top,    SET_UVALUE=state
   END

   'seed1': BEGIN
      state.par.seed1 = event.value
      WIDGET_CONTROL, event.top, SET_UVALUE=state
   END

   'seed2': BEGIN
      state.par.seed2 = event.value
      WIDGET_CONTROL, event.top, SET_UVALUE=state
   END

   'SAVE': BEGIN

      ; Kolmogorov model means infinite wave-front outer-scale L0
      IF (state.par.model EQ 0) THEN state.par.L0=!VALUES.F_INFINITY

      ; first (dummy) checks...
      IF (state.par.length LE 0.) THEN BEGIN
         dummy = DIALOG_MESSAGE("screens' length MUST BE > 0", $
                                DIALOG_PARENT=event.top, TITLE='ATM error', /ERROR)
         RETURN
      ENDIF
      IF (state.par.r0 LE 0.) THEN BEGIN
         dummy = DIALOG_MESSAGE("Fried parameter r0 MUST BE > 0", $
                                DIALOG_PARENT=event.top, TITLE='ATM error', /ERROR)
         RETURN
      ENDIF
      IF (state.par.dim LE 0) THEN BEGIN
         dummy = DIALOG_MESSAGE("screens' dimension MUST BE > 0", $
                                DIALOG_PARENT=event.top, TITLE='ATM error', /ERROR)
         RETURN
      ENDIF
      IF (state.par.model NE 0) THEN BEGIN
         IF (state.par.L0 LE 0) THEN BEGIN
            dummy = DIALOG_MESSAGE("wavefront outer-scale L0 MUST BE > 0", $
                                   DIALOG_PARENT=event.top, TITLE='ATM error', /ERROR)
            RETURN
         ENDIF
      ENDIF
      IF (state.par.cal EQ 0) THEN BEGIN
         IF (state.par.delta_t LE 0) THEN BEGIN
            dummy = DIALOG_MESSAGE("time base MUST BE > 0", $
                                   DIALOG_PARENT=event.top, TITLE='ATM error', /ERROR)
            RETURN
         ENDIF
      ENDIF
      IF (state.par.method EQ 0) THEN BEGIN
         IF (state.par.sha LT 0) THEN BEGIN
            dummy = DIALOG_MESSAGE(["the nb of subharmonics to be added MUST BE >= 0"], $
                                   DIALOG_PARENT=event.top, TITLE='ATM error', /ERROR)
            RETURN
         ENDIF
      ENDIF

      ; check if Cn2(h) sum over altitude is equal to 1
      IF state.par.turnatmos THEN BEGIN
         sum_Cn2 = total(state.par.weight[0:state.par.n_layers-1])
         IF ( abs(sum_Cn2-1) ge 1e-6 ) THEN BEGIN
            dummy = DIALOG_MESSAGE(                                         $
               ['total sum of the relative weights of the',                 $
               'turbulent layers MUST BE 1 (not '+strtrim(sum_Cn2, 2)+')'], $
               DIALOG_PARENT=event.top, TITLE='ATM error', /ERROR)
            RETURN
         ENDIF
      ENDIF

      ; check the wind directions in case of temporal evolution AND use of
      ; already computed STRIPES
      IF ((state.par.cal EQ 0) AND (state.par.lps EQ 0)) THEN BEGIN
         error = psg_openr_cube(unit, state.par.psg_add, header, /GET_LUN)
         FREE_LUN, unit
         IF (header.dim_x NE header.dim_y) THEN BEGIN    ; stripes case
            FOR i = 0, state.par.n_layers-1 DO BEGIN
               IF (state.par.dir[i] NE 0.           $
               and state.par.dir[i] NE 90./!RADEG   $
               and state.par.dir[i] NE 180./!RADEG  $
               and state.par.dir[i] NE 270./!RADEG) $
               THEN BEGIN
                  dummy = DIALOG_MESSAGE(                                  $
                     ['in this CASE, the wind direction MUST BE 0 or',     $
                      '90 deg. (x-dir.), or 180 or 270 deg. (y-dir.)'], $
                     DIALOG_PARENT=event.top, TITLE='ATM error', /ERROR)
                  RETURN
               ENDIF
            ENDFOR
         ENDIF
      ENDIF

      ; check that wind directions are angles (in case of temporal evolution)
      IF (state.par.cal EQ 0) THEN BEGIN
         FOR i = 0, state.par.n_layers-1 DO BEGIN
            IF (state.par.dir[i] LT 0. OR state.par.dir[i] GT 2*!PI) THEN BEGIN
               dummy = DIALOG_MESSAGE('wind directions MUST BE angles between 0 and 360 deg. !!', $
                                      DIALOG_PARENT=event.top, TITLE='ATM error', /ERROR)
               RETURN
            ENDIF
         ENDFOR
      ENDIF

      ; check that wind velocities have positive values (in case of temp. evol.)
      IF (state.par.cal EQ 0) THEN BEGIN
         FOR i = 0, state.par.n_layers-1 DO BEGIN
            IF (state.par.wind[i] lt 0.) THEN BEGIN
               dummy = DIALOG_MESSAGE('wind velocities MUST HAVE positive values', $
                                      DIALOG_PARENT=event.top, TITLE='ATM error', /ERROR)
               RETURN
            ENDIF
         ENDFOR
      ENDIF


      ; do the following checks only if phase screen generation is desired
      IF (state.par.lps EQ 1) THEN BEGIN

            ;; checking that desired number of Zernike polynomials is within range of
            ;; covariance matrix in file given by par.add_covmat
            RESTORE,state.par.add_covmat
            l_dim    = l_sprs.ija[0] - 2
            jmin     = 2L
            sup_jmax = l_dim + jmin - 1             ;Max. Zernike polynomial in cov matrix.
            sup_nmax = LONG(SQRT(2*sup_jmax)-1)     ;Radial order corresponding to sup_jmax.

            IF ((state.par.zern_rad_degree LT 1 ) $
            OR  (state.par.zern_rad_degree GT sup_nmax)) THEN BEGIN 
               dummy =                                                          $
                 DIALOG_MESSAGE(['par specifies that COVARIANCE matrix for the',$
                                 ' Zernike modes is read from file:           ',$
                                 ''                                            ,$
                                 '  '+state.par.add_covmat                     ,$
                                 ''                                            ,$
                                 'for which maximum Zernike radial degree MUST',$
                                 ' BE >= 1 and <= ' + STRCOMPRESS(sup_nmax)],   $
                                 TITLE='ATM error', DIALOG_PARENT=atm_gui,/ERROR)
               state.par.zern_rad_degree = sup_nmax
               WIDGET_CONTROL, state.id.zern_rad_degree, SET_VALUE=state.par.zern_rad_degree
               RETURN
            ENDIF 

            IF (state.par.dim/2 NE state.par.dim/float(2)) THEN BEGIN
               dummy = DIALOG_MESSAGE('phase screens linear nb of pix. MUST BE even', $
                                      DIALOG_PARENT=event.top, TITLE='ATM error', /ERROR)
               RETURN
            ENDIF

      ENDIF

      ; check before saving the parameter file if filename already exists
      check_file = FINDFILE(state.sav_file)
      IF check_file[0] NE "" THEN BEGIN
         dummy = DIALOG_MESSAGE(['file '+state.sav_file+' already exists.', $
                                 ' would you like to overwrite it ?'],      $
                                DIALOG_PARENT=event.top, TITLE='ATM warning', /QUEST)
         IF strlowCASE(dummy) EQ "no" THEN RETURN
      ENDIF ELSE BEGIN
         dummy = DIALOG_MESSAGE(['file '+state.sav_file+' will be SAVEd.'], $
                                DIALOG_PARENT=event.top, TITLE='ATM inFORmation', /INFO)
         ; inform were the parameters will be saved
      ENDELSE

      ; save the parameter data file
      par = state.par
      SAVE, par, FILENAME=state.sav_file

      ; kill the GUI returning a null error
      error = !caos_error.ok
      WIDGET_CONTROL, event.top, /DESTROY

   END

   'help' : online_help, book=(atm_info()).help, /FULL_PATH

   'RESTORE': BEGIN

      ; restore the desired parameter file
      par = 0
      title = "parameter file to RESTORE"
      RESTORE, filename_gui(state.def_file,                          $
                            title,                                   $
                            GROUP_LEADER=event.top,                  $
                            FILTER=state.par.module.mod_name+'*sav', $
                            /NOEDIT,                                 $
                            /MUST_EXIST,                             $
                            /ALL_EVENTS                              )

      ; update the current module number
      par.module.n_module = state.par.module.n_module

      ; set the default values for all the widgets
      WIDGET_CONTROL, state.id.turnatmos, SET_VALUE=par.turnatmos
      WIDGET_CONTROL, state.id.cal,       SET_VALUE=par.cal
      WIDGET_CONTROL, state.id.delta_t,   SET_VALUE=par.delta_t
      WIDGET_CONTROL, state.id.r0,        SET_VALUE=par.r0
      WIDGET_CONTROL, state.id.L0,        SET_VALUE=par.L0
      WIDGET_CONTROL, state.id.n_lay,     SET_VALUE=par.n_layers
      WIDGET_CONTROL, state.id.psg_add,   SET_VALUE=par.psg_add
      WIDGET_CONTROL, state.id.lps,       SET_VALUE=par.lps
      WIDGET_CONTROL, state.id.method,    SET_VALUE=par.method
      WIDGET_CONTROL, state.id.length,    SET_VALUE=par.length
      WIDGET_CONTROL, state.id.width,     SET_VALUE=par.length
      WIDGET_CONTROL, state.id.dim,       SET_VALUE=par.dim
      WIDGET_CONTROL, state.id.model,     SET_VALUE=par.model
      WIDGET_CONTROL, state.id.sam_r0,    SET_VALUE=par.r0*par.dim/par.length
      WIDGET_CONTROL, state.id.fov,       SET_VALUE=5E-7*par.dim/par.length*!RADEG*3600.
      WIDGET_CONTROL, state.id.sha,       SET_VALUE=par.sha
      np = par.n_layers & atm_val = fltarr(4, np)
      atm_val[0,*] = par.alt [0:np-1] & atm_val[1,*] = par.weight[0:np-1]
      atm_val[2,*] = par.wind[0:np-1] & atm_val[3,*] = par.dir   [0:np-1]/!RADEG
      WIDGET_CONTROL, state.id.atm,       SET_VALUE=atm_val

      ; update the state structure
      state.par = par

      ; reset the setting parameters status
      atm_gui_set, state

      ; write the GUI state structure
      WIDGET_CONTROL, event.top, SET_UVALUE=state

   END

   'cancel': BEGIN
      error = !caos_error.cancel
      WIDGET_CONTROL, event.top, /DESTROY
   END

ENDCASE

END

;;;;;;;;;;;;;;;;;;;;;;;
; GUI generation code ;
;;;;;;;;;;;;;;;;;;;;;;;
;
FUNCTION atm_gui, n_module, proj_name, GROUP_LEADER=group

; error status from the event handler procedure
COMMON error_block, error

; retrieve the module information
info = atm_info()

; check if a saved parameter file exists. if it exists it is restored,
; otherwise the default parameter file is restored.
sav_file = mk_par_name(info.mod_name, n_module, PROJ_NAME=proj_name)
def_file = mk_par_name(info.mod_name, PACK_NAME=info.pack_name, /DEFAULT)
par=0
check_file = FINDFILE(sav_file)
IF check_file[0] EQ '' THEN BEGIN
   RESTORE, def_file
   par.module.n_module = n_module
   IF par.module.mod_name NE info.mod_name THEN        $
      message, 'the default parameter file ('+def_file $
              +') is from another module: please take the right one'
   IF (par.module.ver NE info.ver) THEN       $
      message, 'the default parameter file ('+def_file $
              +') is not compatible: please generate it again'
ENDIF ELSE BEGIN
   RESTORE, sav_file
   IF (par.module.mod_name NE info.mod_name) THEN $
      message, 'the parameter file '+sav_file     $
              +' is from another module: please generate a new one'
   IF (par.module.ver NE info.ver) THEN BEGIN
      PRINT, '************************************************************'
      PRINT, 'WARNING: the parameter file '+sav_file
      PRINT, 'is probably from an older version than '+info.pack_name+' !!'
      PRINT, 'You should possibly need to generate it again...'
      PRINT, '************************************************************'
   ENDIF
ENDELSE

atm_val      = fltarr(4, par.n_layers)
atm_val[0,*] = par.alt   [0:par.n_layers-1]
atm_val[1,*] = par.weight[0:par.n_layers-1]
atm_val[2,*] = par.wind  [0:par.n_layers-1]
atm_val[3,*] = par.dir   [0:par.n_layers-1]/!RADEG

id = $
   { $
   turnatmos      : 0L, $
   par_base       : 0L, $
   cal            : 0L, $
   delta_t        : 0L, $
   evol_t         : 0L, $
   t0_tab         : 0L, $
   r0             : 0L, $
   r0_tab         : 0L, $
   L0             : 0L, $
   n_lay          : 0L, $
   atm            : 0L, $
   lps            : 0L, $
   psg            : 0L, $
   read_ps        : 0L, $
   psg_add        : 0L, $
   psg_nscreens   : 0L, $
   psg_dimx       : 0L, $
   psg_dimy       : 0L, $
   psg_L0         : 0L, $
   psg_model      : 0L, $
   psg_meth       : 0L, $
   psg_sha        : 0L, $
   psg_prec       : 0L, $
   psg_seed1      : 0L, $
   psg_seed2      : 0L, $
   method         : 0L, $
   length         : 0L, $
   width          : 0L, $
   dim            : 0L, $
   model          : 0L, $
   sam_r0         : 0L, $
   sam_r0_tab     : 0L, $
   fov            : 0L, $
   fov_tab        : 0L, $
   rec_sha        : 0L, $
   sha            : 0L, $
   zern_rad_degree: 0L, $
   title_sha      : 0L, $
   sha_test       : 0L, $
   seed1          : 0L, $
   seed2          : 0L  $
   }

state = $
   {    $
   sav_file: sav_file, $
   def_file: def_file, $
   id      : id,       $
   par     : par       $
   }

; root base
modal = N_ELEMENTS(group) NE 0
title = STRUPCASE(info.mod_name)+" parameters setting GUI"
root_base_id = WIDGET_BASE(TITLE=title, MODAL=modal, /COL, GROUP_LEADER=group)
WIDGET_CONTROL, root_base_id, SET_UVALUE=state
   
   ; parameters base
   state.id.par_base = WIDGET_BASE(root_base_id, FRAME=10, ROW=2)

   base = WIDGET_BASE(state.id.par_base,  /COLUMN)
   text = WIDGET_LABEL(base, VALUE=' ATMosphere modelling module ', /FRAME)
   
   ; 1st-TAB base
   basetab = WIDGET_TAB(state.id.par_base, location=0, uvalue='')
   basetabA = WIDGET_BASE(basetab, /ROW, TITLE="     ATMOSPHERE+LAYERS' SCREENS PARAMETERS     ")

   ; 1st-TAB left-part base
   left_base_id = WIDGET_BASE(basetabA, /COL)

      ; turn on/turn off base
      turn_base_id = WIDGET_BASE(left_base_id, COL=2, /FRAME)

         dummy = WIDGET_LABEL(turn_base_id, VALUE="turbulent atmosphere control", /FRAME)

         state.id.turnatmos = CW_BGROUP(turn_base_id, ['turn off !','turn on !'],          $
                                        SET_VALUE=state.par.turnatmos, UVALUE='turnatmos', $
                                        COLUMN=2, /EXCLUSIVE)

      ; temp.evol./stat.averaging base
      cal_base_id = WIDGET_BASE(left_base_id, ROW=2, /FRAME)

      dummy = WIDGET_BASE(cal_base_id, COL=2)
         dum = WIDGET_LABEL(dummy, VALUE='type of calculation', /FRAME)

         state.id.cal = CW_BGROUP(dummy,               $
            ['temporal evolution', 'statistical averaging'], $
            SET_VALUE=state.par.cal, UVALUE='menu_cal', COLUMN=2, /EXCLUSIVE)

         dummy = WIDGET_BASE(cal_base_id, /ROW)
         
            state.id.delta_t = CW_FIELD(dummy,            $
               TITLE='time-base [s]', /COLUMN,            $
               VALUE=state.par.delta_t, UVALUE='delta_t', $
               /FLOATING, /ALL_EVENTS)
               
            state.id.evol_t = CW_FIELD(dummy,                       $
                                       TITLE='(tau0 [s] (@500nm))', $
                                       /COLUMN, /NOEDIT, /FLOATING  )

            state.id.t0_tab = WIDGET_BUTTON(dummy,                         $
                                         VALUE="(tau0 vs. lambda)", $
                                         UVALUE="t0_tab"                    )

      ; atm'c turb. parameters' base
      atm_base_id = WIDGET_BASE(left_base_id, /FRAME, ROW=6)

         dummy = WIDGET_LABEL(atm_base_id, $
            VALUE='turbulent atmosphere parameters', /FRAME)

         lay_base_id = WIDGET_BASE(atm_base_id, /ROW)
         
            state.id.n_lay = CW_FIELD(lay_base_id,       $
               TITLE="nb of layers (1-10)",    $
               VALUE=state.par.n_layers, UVALUE='n_lay', $
               /INTEGER, /RETURN_EVENTS, /COL)
               
         state.id.r0 = CW_FIELD(lay_base_id, $
            TITLE="r0 [m] (@500nm): ", $
            VALUE=state.par.r0, UVALUE='r0', /COL, /FLOATING, /ALL_EVENTS)

         state.id.r0_tab = WIDGET_BUTTON(lay_base_id,                   $
                                         VALUE="(r0 vs. lambda)", $
                                         UVALUE="r0_tab"          )

         dummy = WIDGET_LABEL(atm_base_id, VALUE='characteristics of the turbulent layers')

         state.id.atm = WIDGET_TABLE(atm_base_id, $
            ROW_LABELS='layer #'+strtrim(indgen(state.par.n_layers),2),   $
            COLUMN_LABELS=["h [m]", "Cn2 ratio", "v [m/s]", "dir [deg]"], $
            VALUE=atm_val, UVALUE='table_atm', /EDITABLE, YSIZE=9         )
		 
   ; 1st-TAB right-part base
   right_base_id = WIDGET_BASE(basetabA, /COL)

      ; layers base
      layers_base_id = WIDGET_BASE(right_base_id, /FRAME, ROW=4)

         dummy = WIDGET_LABEL(layers_base_id,                $
                              VALUE="layers' phase screens", $
                              /FRAME)

		 dummy = WIDGET_BASE(layers_base_id, COL=3)
		  
         state.id.lps = CW_BGROUP(dummy,                                 $
                                  ['use already computed phase screens', $
                                   'generate new ones'],                 $
                                  SET_VALUE=state.par.lps,               $
                                  UVALUE='menu_lps',                     $
                                  /COLUMN,                               $
                                  /EXCLUSIVE                             )

           state.id.length = CW_FIELD(dummy,                                $
                                      TITLE="screens' physical length [m]", $
                                      /COLUMN,                              $
                                      VALUE=state.par.length,               $
                                      UVALUE='length',                      $
                                      /FLOATING,                            $
                                      /ALL_EVENTS                           )

             state.id.width = CW_FIELD(dummy,                                 $
                                       TITLE="(screens' physical width [m])", $
                                       /COLUMN,                               $
                                       VALUE=state.par.length,                $
                                       /FLOATING,                             $
                                       /NOEDIT                                )

         dummy = WIDGET_BASE(layers_base_id, /ROW)

            state.id.sam_r0 = CW_FIELD(dummy,                                 $
                                       TITLE="(r0 sampling [px/r0] (@500nm))",$
                                       /COLUMN,                               $
                                       VALUE=state.par.r0*state.par.dim/state.par.length,$
                                       /FLOATING,                             $
                                       /NOEDIT                                )

            state.id.sam_r0_tab = WIDGET_BUTTON(dummy,                        $
                                             VALUE="(r0 sampling vs. lambda)",$
                                             UVALUE="sam_r0_tab"              )

         
            state.id.fov= CW_FIELD(dummy,                                     $
                                   TITLE="(FoV [''] (@500nm))",               $
                                   /COLUMN,                                   $
                                   VALUE=5E-7*state.par.dim/state.par.length*!RADEG*3600.,$
                                   /FLOATING,                                 $
                                   /NOEDIT                                    )

            state.id.fov_tab = WIDGET_BUTTON(dummy,                         $
                                             VALUE="(FoV vs. lambda)", $
                                             UVALUE="fov_tab"               )

      ; already-comp'd-screens base
      state.id.read_ps = WIDGET_BASE(right_base_id, /FRAME, ROW=6)

         dummy = WIDGET_LABEL(state.id.read_ps, VALUE='phase screens file', /FRAME)

         state.id.psg_add = CW_FILENAME(state.id.read_ps,                   $
                                        TITLE="phase screens file address", $
                                        VALUE=state.par.psg_add,            $
                                        UVALUE='psg_add',                   $
                                        /RETURN_EVENTS, XSIZE=90            )

         dummy = WIDGET_BASE(state.id.read_ps, COL=4)
         
            state.id.psg_dimx = CW_FIELD(dummy,                         $
                                         TITLE="(screens' x-dim [px])", $
                                         /COLUMN,                       $
                                         /INTEGER,                      $
                                         /NOEDIT                        )
                                         
            state.id.psg_dimy = CW_FIELD(dummy,                         $
                                         TITLE="(screens' y-dim [px])", $
                                         /COLUMN,                       $
                                         /INTEGER,                      $
                                         /NOEDIT                        )
                                         
            state.id.psg_nscreens = CW_FIELD(dummy,                   $
                                             TITLE="(nb of screens)", $
                                             /COLUMN,                 $
                                             /INTEGER,                $
                                             /NOEDIT                  )
                                             
            state.id.psg_meth = CW_FIELD(dummy,                      $
                                         TITLE="(computing method)", $
                                         /COLUMN,                    $
                                         /NOEDIT                     )

         dummy = WIDGET_BASE(state.id.read_ps, COL=4)
         
            state.id.psg_model = CW_FIELD(dummy,                       $
                                          TITLE="(atmospheric model)", $
                                          /COLUMN,                     $
                                          /NOEDIT                      )
                                          
            state.id.psg_sha = CW_FIELD(dummy,                              $
                                        TITLE="(nb of subharmonics added)", $
                                        /COLUMN,                            $
                                        /INTEGER,                           $
                                        /NOEDIT                             )
                                        
           state.id.psg_prec = CW_FIELD(dummy,               $
                                        TITLE="(precision)", $
                                        /COLUMN,             $
                                        /NOEDIT              )
                                        
            state.id.psg_seed1= CW_FIELD(dummy,                      $
                                         TITLE="(FFT/Zernike seed)", $
                                         /COLUMN,                    $
                                         /INTEGER,                   $
                                         /NOEDIT                     )

         dummy = WIDGET_BASE(state.id.read_ps, COL=4)
         
             state.id.psg_seed2 = CW_FIELD(dummy,                      $
                                          TITLE="(subharmonics seed)", $
                                          /COLUMN,                     $
                                          /INTEGER,                    $
                                          /NOEDIT                      )

         state.id.psg_L0 = CW_FIELD(dummy,                             $
                                    TITLE="(wf outer scale L0 [m]): ", $
                                    /FLOATING,                         $
                                    /NOEDIT                            )

      ; please-see-next-tab base
      dummy = WIDGET_BASE(right_base_id, /FRAME, /ROW)

         dummy = WIDGET_LABEL(dummy,                                      $
                              VALUE="PLEASE ALSO SEE NEXT TAB FOR 'PHASE" $
                                   +" SCREEN GENERATION' PARAMETERS...",  $
                              /FRAME                                      )


   ; 2nd-TAB base
   basetabB=WIDGET_BASE(basetab, /COL, TITLE='     PHASE SCREEN GENERATION     ')
   base_stopping_rule = WIDGET_BASE(basetabB, /FRAME, /COL)

      ; phase-screen-generation base
      state.id.psg = WIDGET_BASE(base_stopping_rule, /FRAME, ROW=4)

         dummy = WIDGET_LABEL(state.id.psg,                    $
                              VALUE='phase screen generation', $
                              /FRAME                           )

         dummy = WIDGET_BASE(state.id.psg, COLUMN=6)

            state.id.method = CW_BGROUP(dummy,                        $
                                        LABEL_TOP='computing method', $
                                        ['FFT (+ subharmonics)',      $
                                         'Zernike polynomials'],      $
                                        SET_VALUE=state.par.method,   $
                                        UVALUE='menu_meth',           $
                                        /EXCLUSIVE                    )

            state.id.model = CW_BGROUP(dummy,                         $
                                       LABEL_TOP="atmospheric model", $
                                       ['Kolmogorov', 'von Karman'],  $
                                       SET_VALUE=state.par.model,     $
                                       UVALUE='menu_mod',             $
                                       /EXCLUSIVE                     )

            dumdummy = WIDGET_BASE(dummy, ROW=2)

               state.id.seed1 = CW_FIELD(dumdummy,                    $
                                         TITLE="FFT/Zernike seed",    $
                                         /COLUMN,                     $
                                         VALUE=state.par.seed1,       $
                                         /LONG,                       $
                                         UVALUE='seed1',              $
                                         /ALL_EVENTS                  )

               state.id.seed2 = CW_FIELD(dumdummy,                    $
                                         TITLE="subharmonics seed",   $
                                         /COLUMN,                     $
                                         VALUE=state.par.seed2,       $
                                         /LONG,                       $
                                         UVALUE='seed2',              $
                                         /ALL_EVENTS                  )
                                         
            dummy_base = WIDGET_BASE(dummy, ROW=3)

               dumdummy = WIDGET_BASE(dummy_base, ROW=2)

              state.id.title_sha = WIDGET_LABEL(dumdummy,                  $
                                                VALUE='nb of subharmonics' )

                  state.id.sha_test = WIDGET_BUTTON(dumdummy,               $
                     VALUE='PUSH HERE to test subharmonics adding accuracy !!',$
                     UVALUE='sha_test'                                        )


                dumdummy = WIDGET_BASE(dummy_base, COL=2)

                  state.id.rec_sha = CW_FIELD(dumdummy,              $
                                              TITLE='(recommENDed)', $
                                              /COLUMN,               $
                                              VALUE=state.par.sha,   $
                                              UVALUE='rec_sha',      $
                                              /NOEDIT,               $
                                              XSIZE=3                )

                  state.id.sha     = CW_FIELD(dumdummy,              $
                                              TITLE='desired',       $
                                              /COLUMN,               $
                                              VALUE=state.par.sha,   $
                                              /INTEGER,              $
                                              UVALUE='sha',          $
                                              /ALL_EVENTS,           $
                                              XSIZE=3                )

            dummy_base = WIDGET_BASE(dummy, ROW=2)

                  state.id.dim = CW_FIELD(dummy_base,                      $
                                          TITLE='x- and y-dimension [px]', $
                                          /COLUMN,                         $
                                          VALUE=state.par.dim,             $
                                          UVALUE='dim',                    $
                                          /INTEGER,                        $
                                          /ALL_EVENTS                      )

                  state.id.L0 = CW_FIELD(dummy_base,                       $
                                         TITLE="wf outer scale L0 [m]",    $
                                         /COLUMN,                          $
                                         VALUE=state.par.L0,               $
                                         /FLOATING,                        $
                                         UVALUE='L0',                      $
                                         /ALL_EVENTS                       )

             state.id.zern_rad_degree = CW_FIELD(dummy,                             $
                                                 TITLE='max. Zernike radial degree',$
                                                 VALUE=state.par.zern_rad_degree,   $
                                                 /INTEGER,                          $
                                                 UVALUE='zern_rad_degree',          $
                                                 /ALL_EVENTS,                       $
                                                 /COLUMN                            )

   ; final widget (notes+buttons)
   dummy = WIDGET_BASE(root_base_id, /ROW)
   
   ; base for footnote
   note_base_id = WIDGET_BASE(dummy, /COL, FRAME=10)
      dum = ['NOTE:'                                                       $
            +' Parameters with a title in between brackets - "(title)" and'$
            +' not "title" - are here FYI, they have non-editable fields.' ]
      dum = WIDGET_TEXT(note_base_id, VALUE=dum, YSIZE=N_ELEMENTS(dum))

   ; button base for control buttons (standard buttons)
   btn_base_id = WIDGET_BASE(dummy, FRAME=10, /ROW)
      dummy = WIDGET_BUTTON(btn_base_id, VALUE="HELP", UVALUE="help")
      cancel_id = WIDGET_BUTTON(btn_base_id, VALUE="CANCEL", UVALUE="cancel")
      IF modal THEN WIDGET_CONTROL, cancel_id, /CANCEL_BUTTON
      dummy = WIDGET_BUTTON(btn_base_id, VALUE="RESTORE PARAMETERS", $
                            UVALUE="RESTORE")
      SAVE_id = WIDGET_BUTTON(btn_base_id, VALUE="SAVE PARAMETERS", $
                              UVALUE="SAVE")
      IF modal THEN WIDGET_CONTROL, SAVE_id, /DEFAULT_BUTTON

; initialize all the sensitive states
atm_gui_set, state

; SAVE the state structure of the GUI in the top base uvalue
WIDGET_CONTROL, root_base_id, SET_UVALUE=state

; draw the GUI
WIDGET_CONTROL, root_base_id, /REALIZE

XMANAGER, 'atm_gui', root_base_id, GROUP_LEADER=group

RETURN, error
END